"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = slackin;

var _express = _interopRequireDefault(require("express"));

var _socket = _interopRequireDefault(require("socket.io"));

var _bodyParser = require("body-parser");

var _http = require("http");

var _fs = require("fs");

var _emailRegex = _interopRequireDefault(require("email-regex"));

var _cors = _interopRequireDefault(require("cors"));

var _superagent = _interopRequireDefault(require("superagent"));

var _debug = _interopRequireDefault(require("debug"));

var _tinycolor = _interopRequireDefault(require("tinycolor2"));

var _micromatch = _interopRequireDefault(require("micromatch"));

var _slack = _interopRequireDefault(require("./slack"));

var _slackInvite = _interopRequireDefault(require("./slack-invite"));

var _themes = _interopRequireDefault(require("./themes"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// their code
// our code
var mainLog = (0, _debug["default"])('slackin:main');
var inviteLog = (0, _debug["default"])('slackin:invite');
var slackLog = (0, _debug["default"])('slackin:slack');

function slackin(_ref) {
  var token = _ref.token,
      org = _ref.org,
      _ref$path = _ref.path,
      path = _ref$path === void 0 ? '/' : _ref$path,
      _ref$interval = _ref.interval,
      interval = _ref$interval === void 0 ? 60000 : _ref$interval,
      _ref$cors = _ref.cors,
      useCors = _ref$cors === void 0 ? false : _ref$cors,
      _ref$recaptcha = _ref.recaptcha,
      recaptcha = _ref$recaptcha === void 0 ? {} : _ref$recaptcha,
      analytics = _ref.analytics,
      themeID = _ref.theme,
      accent = _ref.accent,
      css = _ref.css,
      channelFilter = _ref.channels,
      emails = _ref.emails,
      coc = _ref.coc,
      proxy = _ref.proxy,
      redirectFQDN = _ref.redirectFQDN,
      letsencrypt = _ref.letsencrypt,
      silent = _ref.silent,
      server = _ref.server;
  // must haves
  if (!token) throw new Error('Must provide a `token`.');
  if (!org) throw new Error('Must provide an `org`.');

  if (!!(recaptcha.secret || recaptcha.sitekey || recaptcha.invisible) !== !!(recaptcha.secret && recaptcha.sitekey)) {
    throw new Error('Both `recaptcha-secret` and `recaptcha-sitekey` must be defined to enable reCAPTCHA.');
  }

  var relativePath;

  if (path.slice(-1) !== '/') {
    relativePath = "".concat(path, "/");
  } else {
    relativePath = path;
  }

  if (!silent) {
    inviteLog.enabled = true;
    slackLog.enabled = true;
    mainLog.enabled = true;
  }

  var channels;

  if (channelFilter) {
    channels = channels.split(',').map(function (channel) {
      return channel[0] === '#' ? channel.substr(1) : channel;
    });
  }

  var acceptedEmails;

  if (emails) {
    acceptedEmails = emailFilter.split(',');
  }

  var theme;

  if (themeID) {
    if (themeID in _themes["default"]) {
      theme = _themes["default"][themeID];
    } else {
      mainLog("Specified theme (".concat(themeID, ") not found, falling back to default"));
    }
  } else {
    theme = _themes["default"]["default"];
  }

  mainLog("Theme: ".concat(theme.name));

  if (accent) {
    theme.accent = (0, _tinycolor["default"])(accent).toHexString();
    mainLog("Using a custom theme accent: ".concat(accent));
  }

  theme.accentDark = (0, _tinycolor["default"])(theme.accent).darken(10).toHexString(); // setup

  var app = (0, _express["default"])();
  var srv = server || (0, _http.Server)(app);
  srv.app = app;
  app.set('views', "".concat(__dirname, "/../views"));
  app.set('view engine', 'pug');

  if (useCors) {
    app.options('*', (0, _cors["default"])());
    app.use((0, _cors["default"])());
  }

  if (proxy) {
    app.enable('trust proxy');

    if (redirectFQDN) {
      app.use(function (req, res, next) {
        if (req.headers['x-forwarded-proto'] === 'http') {
          res.redirect("https://".concat(redirectFQDN).concat(req.url));
        } else {
          next();
        }
      });
    }
  } // static files


  var assets = "".concat(__dirname, "/assets");
  app.use('/assets', _express["default"]["static"](assets));
  app.use('/slackin.js', _express["default"]["static"]("".concat(assets, "/badge.js")));

  if (letsencrypt) {
    app.get('/.well-known/acme-challenge/:id', function (req, res) {
      res.send(letsencrypt);
    });
  } // fetch data


  mainLog('Establishing connection with Slack');
  var slack = new _slack["default"]({
    token: token,
    interval: interval,
    org: org,
    logger: slackLog
  });
  slack.setMaxListeners(Infinity); // middleware for waiting for slack

  app.use(function (req, res, next) {
    if (slack.ready) return next();
    return slack.once('ready', next);
  });
  app.get('/', function (req, res) {
    var _slack$org = slack.org,
        name = _slack$org.name,
        logo = _slack$org.logo;
    var _slack$users = slack.users,
        active = _slack$users.active,
        total = _slack$users.total;
    if (!name) return res.send(404);
    return res.type('html').render('main', {
      coc: coc,
      path: relativePath,
      name: name,
      org: org,
      logo: logo,
      active: active,
      total: total,
      recaptcha: recaptcha,
      css: css,
      analytics: analytics,
      channels: channels,
      theme: theme
    });
  });
  app.get('/data', function (req, res) {
    var _slack$org2 = slack.org,
        name = _slack$org2.name,
        logo = _slack$org2.logo;
    var _slack$users2 = slack.users,
        active = _slack$users2.active,
        total = _slack$users2.total;
    res.send({
      name: name,
      org: org,
      coc: coc,
      logo: logo,
      active: active,
      total: total,
      channels: channels
    });
  }); // invite endpoint

  app.post('/invite', (0, _bodyParser.json)(), function (req, res) {
    var _req$body = req.body,
        channel = _req$body.channel,
        email = _req$body.email;
    var captchaResponse = req.body['g-recaptcha-response'];
    var errorMessage = null;

    if (channels && !channels.includes(channel)) {
      errorMessage = 'Not a permitted channel';
    } else if (channels && !slack.getChannelId(channel)) {
      errorMessage = "Channel \"".concat(channel, "\" not found");
    } else if (!email) {
      errorMessage = 'No email provided';
    } else if (recaptcha.secret && (!captchaResponse || !captchaResponse.length)) {
      errorMessage = 'Invalid captcha';
    } else if (!(0, _emailRegex["default"])().test(email)) {
      errorMessage = 'Invalid email';
    } else if (emails && !_micromatch["default"].any(email, acceptedEmails)) {
      errorMessage = 'Your email is not on the accepted list.';
    } else if (coc && req.body.coc != 1) {
      errorMessage = 'Agreement to CoC is mandatory';
    }

    if (errorMessage) {
      return res.status(400).json({
        msg: errorMessage
      });
    }

    var captchaData = {
      secret: recaptcha.secret,
      response: captchaResponse,
      remoteip: req.connection.remoteAddress
    };

    var captchaCallback = function captchaCallback(err) {
      if (err) return res.status(400).send({
        msg: err
      });
      return (0, _slackInvite["default"])({
        token: token,
        org: org,
        email: email,
        logger: inviteLog,
        channel: slack.channel ? slack.channel.id : null
      }, function (inviteErr) {
        if (inviteErr) {
          if (inviteErr.message === 'Sending you to Slack...') {
            return res.status(303).json({
              msg: inviteErr.message,
              redirectUrl: "https://".concat(org, ".slack.com")
            });
          }

          return res.status(400).json({
            msg: inviteErr.message
          });
        }

        return res.status(200).json({
          msg: 'WOOT. Check your email!'
        });
      });
    };

    if (recaptcha.secret) {
      return _superagent["default"].post('https://www.google.com/recaptcha/api/siteverify').type('form').send(captchaData).end(captchaCallback);
    }

    return captchaCallback();
  }); // iframe

  app.get('/iframe', function (req, res) {
    var logo = (0, _fs.readFileSync)("".concat(__dirname, "/assets/slack.svg")).toString('base64');
    var js = (0, _fs.readFileSync)("".concat(__dirname, "/assets/iframe.js")).toString();
    var extraCss = (0, _fs.readFileSync)("".concat(__dirname, "/assets/iframe-button.css")).toString();
    var large = 'large' in req.query;
    var _slack$users3 = slack.users,
        active = _slack$users3.active,
        total = _slack$users3.total;
    res.type('html');
    res.render('iframe', {
      path: relativePath,
      active: active,
      total: total,
      large: large,
      logo: logo,
      js: js,
      extraCss: extraCss,
      css: css
    });
  });
  app.get('/iframe/dialog', function (req, res) {
    var large = 'large' in req.query;
    var name = slack.org.name;
    var _slack$users4 = slack.users,
        active = _slack$users4.active,
        total = _slack$users4.total;

    if (!name) {
      res.sendStatus(404);
      return;
    }

    res.type('html');
    res.render('main', {
      coc: coc,
      path: relativePath,
      name: name,
      org: org,
      active: active,
      total: total,
      large: large,
      recaptcha: recaptcha,
      analytics: analytics,
      channels: channels,
      theme: theme,
      iframe: true
    });
  }); // badge rendering

  app.get('/badge.svg', function (req, res) {
    res.type('svg');
    res.set('Cache-Control', 'max-age=0, no-cache');
    res.set('Pragma', 'no-cache');
    var options = {
      total: slack.users.total,
      active: slack.users.active,
      bg: req.query.bg ? (0, _tinycolor["default"])(req.query.bg).toHexString() : theme.accent
    };

    if (req.query.fg) {
      options.fg = (0, _tinycolor["default"])(req.query.fg).toHexString();
    } else {
      options.fg = (0, _tinycolor["default"])(options.bg).isDark() ? '#fff' : '#333';
    }

    res.render('badge-svg', options);
  }); // realtime

  (0, _socket["default"])(srv, {
    path: "".concat(relativePath, "socket.io")
  }).on('connection', function (socket) {
    var change = function change(key, val) {
      return socket.emit(key, val);
    };

    slack.on('change', change);
    socket.emit('data', slack.users);
    socket.on('disconnect', function () {
      slack.removeListener('change', change);
    });
  });
  return srv;
}