"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = invite;

var _superagent = _interopRequireDefault(require("superagent"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function invite(_ref, fn) {
  var org = _ref.org,
      token = _ref.token,
      email = _ref.email,
      channel = _ref.channel,
      logger = _ref.logger;
  var data = {
    email: email,
    token: token
  };

  if (channel) {
    data.channels = channel;
    data.ultra_restricted = 1;
    data.set_active = true;
  }

  _superagent["default"].post("https://".concat(org, ".slack.com/api/users.admin.invite")).type('form').send(data).end(function (err, res) {
    if (err) return fn(err);

    if (res.status !== 200) {
      return fn(new Error("Invalid response ".concat(res.status, ".")));
    } // If the account that owns the token is not admin, Slack will oddly
    // return `200 OK`, and provide other information in the body. So we
    // need to check for the correct account scope and call the callback
    // with an error if it's not high enough.


    var _res$body = res.body,
        ok = _res$body.ok,
        providedError = _res$body.error,
        needed = _res$body.needed;

    if (!ok) {
      if (logger) logger("Error sending an invite to ".concat(email, ": ").concat(providedError));

      if (providedError === 'missing_scope' && needed === 'admin') {
        return fn(new Error('Missing admin scope: The token you provided is for an account that is not an admin. You must provide a token from an admin account in order to invite users through the Slack API.'));
      } else if (providedError === 'already_invited') {
        return fn(new Error('You have already been invited to Slack. Check for an email from feedback@slack.com.'));
      } else if (providedError === 'already_in_team') {
        return fn(new Error('Sending you to Slack...'));
      }

      return fn(new Error(providedError));
    }

    if (logger) logger("Sent an invite to ".concat(email));
    return fn(null);
  });
}