"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _superagent = _interopRequireDefault(require("superagent"));

var _events = require("events");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var SlackData =
/*#__PURE__*/
function (_EventEmitter) {
  _inherits(SlackData, _EventEmitter);

  function SlackData(_ref) {
    var _this;

    var token = _ref.token,
        interval = _ref.interval,
        logger = _ref.logger,
        host = _ref.org;

    _classCallCheck(this, SlackData);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(SlackData).call(this));
    _this.host = host;
    _this.token = token;
    _this.interval = interval;
    _this.ready = false;
    _this.org = {};
    _this.users = {};
    _this.channelsByName = {};

    if (logger) {
      _this.bindLogs(logger);
    }

    _this.init();

    _this.fetch();

    return _this;
  }

  _createClass(SlackData, [{
    key: "init",
    value: function init() {
      var _this2 = this;

      _superagent["default"].get("https://".concat(this.host, ".slack.com/api/channels.list")).query({
        token: this.token
      }).end(function (err, res) {
        if (err) {
          throw err;
        }

        (res.body.channels || []).forEach(function (channel) {
          _this2.channelsByName[channel.name] = channel;
        });
      });

      _superagent["default"].get("https://".concat(this.host, ".slack.com/api/team.info")).query({
        token: this.token
      }).end(function (err, res) {
        var team = res.body.team;

        if (!team) {
          throw new Error('Bad Slack response. Make sure the team name and API keys are correct');
        }

        _this2.org.name = team.name;

        if (!team.icon.image_default) {
          _this2.org.logo = team.icon.image_132;
        }
      });
    }
  }, {
    key: "fetch",
    value: function fetch() {
      var _this3 = this;

      _superagent["default"].get("https://".concat(this.host, ".slack.com/api/users.list")).query({
        token: this.token,
        presence: 1
      }).end(function (err, res) {
        _this3.onres(err, res);
      });

      this.emit('fetch');
    }
  }, {
    key: "getChannelId",
    value: function getChannelId(name) {
      var channel = this.channelsByName[name];
      return channel ? channel.id : null;
    }
  }, {
    key: "retry",
    value: function retry() {
      var delay = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.interval * 2;
      setTimeout(this.fetch.bind(this), delay);
      this.emit('retry');
    }
  }, {
    key: "onres",
    value: function onres(err, res) {
      if (err) {
        this.emit('error', err);
        return this.retry();
      } // Too Many Requests


      if (res.status === 429) {
        return this.retry(res.headers['retry-after'] * 1000);
      }

      var users = res.body.members;

      if (!users || users && !users.length) {
        this.emit('error', new Error("Invalid Slack response: ".concat(res.status)));
        return this.retry();
      } // remove slackbot and bots from users
      // slackbot is not a bot, go figure!


      users = users.filter(function (x) {
        return x.id !== 'USLACKBOT' && !x.is_bot && !x.deleted;
      });
      var total = users.length;
      var active = users.filter(function (user) {
        return user.presence === 'active';
      }).length;

      if (this.users) {
        if (total !== this.users.total) {
          this.emit('change', 'total', total);
        }

        if (active !== this.users.active) {
          this.emit('change', 'active', active);
        }
      }

      this.users.total = total;
      this.users.active = active;

      if (!this.ready) {
        this.ready = true;
        this.emit('ready');
      }

      setTimeout(this.fetch.bind(this), this.interval);
      return this.emit('data');
    }
  }, {
    key: "bindLogs",
    value: function bindLogs(logger) {
      var _this4 = this;

      this.on('error', function (err) {
        return logger('Error: %s', err.stack);
      });
      this.on('retry', function () {
        return logger('Attempt failed, will retry');
      });
      this.on('fetch', function () {
        return logger('Fetching data from Slack');
      });
      this.on('ready', function () {
        logger('Slack is ready');

        if (!_this4.org.logo) {
          logger('Error: No logo exists for the Slack organization.');
        }
      });
      this.on('data', function () {
        return logger('Got data from Slack: %d online, %d total', _this4.users.active, _this4.users.total);
      });
    }
  }]);

  return SlackData;
}(_events.EventEmitter);

exports["default"] = SlackData;